@@ -25,16 +25,7 @@ from . import bflb_utils
 from . import bflb_configobj
 from . import bflb_fdt
 
-try:
-    import changeconf as cgc
-    conf_sign = True
-except ImportError:
-    conf_sign = False
-    
-if conf_sign:
-    from . import bl602
-else:
-    from . import bl60x
-    from . import bl602
-    from . import bl702
-
+from . import bl60x
+from . import bl602
+from . import bl702
+from . import bl808
