@@ -26,6 +26,7 @@ import time
 import binascii
 import serial
 import threading
+import traceback
 from queue import Queue
 from queue import Empty
 
@@ -40,29 +41,38 @@ class BflbUartPort(object):
         self._ser = None
         self._shakehand_flag = False
         self._chiptype = "bl602"
+        self._chipname = "bl602"
 
-    def if_init(self, device, rate, chiptype="bl602"):
+    def if_init(self, device, rate, chiptype="bl602", chipname="bl602"):
         if self._ser is None:
             self._baudrate = rate
-            
+
             if " (" in device:
-                    dev = device[:device.find(" (")]
+                dev = device[:device.find(" (")]
             else:
                 dev = device
             self._device = dev.upper()
-            
-            self._ser = serial.Serial(dev,
-                                      rate,
-                                      timeout=2.0,
-                                      xonxoff=False,
-                                      rtscts=False,
-                                      write_timeout=None,
-                                      dsrdtr=False)
+
+            for i in range(5):
+                try:
+                    self._ser = serial.Serial(dev,
+                                              rate,
+                                              timeout=2.0,
+                                              xonxoff=False,
+                                              rtscts=False,
+                                              write_timeout=None,
+                                              dsrdtr=False)
+                except Exception as error:
+                    bflb_utils.printf(error)
+                    time.sleep(1)
+                else:
+                    break
         else:
             self._ser.baudrate = rate
             self._baudrate = rate
         self._602a0_dln_fix = False
         self._chiptype = chiptype
+        self._chipname = chipname
 
     def if_set_rx_timeout(self, val):
         self._ser.timeout = val
@@ -114,9 +124,11 @@ class BflbUartPort(object):
             if self._shakehand_flag is True:
                 break
             if self._chiptype == "bl702":
-                self._ser.write(self._if_get_sync_bytes(int(0.003 * self._baudrate / 10)))
+                self._ser.write(self._if_get_sync_bytes(
+                    int(0.003 * self._baudrate / 10)))
             else:
-                self._ser.write(self._if_get_sync_bytes(int(0.006 * self._baudrate / 10)))
+                self._ser.write(self._if_get_sync_bytes(
+                    int(0.006 * self._baudrate / 10)))
 
     def bl_usb_serial_write(self, cutoff_time, reset_revert):
         boot_revert = 0
@@ -128,7 +140,7 @@ class BflbUartPort(object):
         data = bytearray()
         specialstr = bflb_utils.string_to_bytearray("BOUFFALOLAB5555RESET")
         for b in specialstr:
-                data.append(b)
+            data.append(b)
         data.append(boot_revert)
         data.append(reset_revert)
         self._ser.write(data)
@@ -145,8 +157,8 @@ class BflbUartPort(object):
             ports = []
             for p, d, h in comports():
                 if "Virtual" in d or not p:
-                    continue 
-                if "PID=FFFF" in h.upper():
+                    continue
+                if "PID=FFFF" in h.upper() or "PID=42BF:B210" in h.upper():
                     if p.upper() == dev.upper():
                         return True
         elif sys.platform.startswith('linux') or sys.platform.startswith('darwin'):
@@ -154,7 +166,7 @@ class BflbUartPort(object):
             for p, d, h in comports():
                 if not p:
                     continue
-                if "PID=FFFF" in h.upper():
+                if "PID=FFFF" in h.upper() or "PID=42BF:B210" in h.upper():
                     if p.upper() == dev.upper():
                         return True
         else:
@@ -169,12 +181,12 @@ class BflbUartPort(object):
                      reset_revert=True,
                      cutoff_time=0,
                      shake_hand_retry=2,
-                     boot2_load=0,
+                     iap_timeout=0,
                      boot_load=False):
         timeout = self._ser.timeout
         blusbserialwriteflag = False
-        if boot2_load > 0:
-            wait_timeout = boot2_load
+        if iap_timeout > 0:
+            wait_timeout = iap_timeout
             self._ser.timeout = 0.1
             self._shakehand_flag = False
             fl_thrx = None
@@ -183,11 +195,11 @@ class BflbUartPort(object):
             fl_thrx.start()
             time_stamp = time.time()
             while time.time() - time_stamp < wait_timeout:
-                success, ack = self.if_read(16)
+                success, ack = self.if_read(3000)
                 if ack.find(b'\x4F') != -1 or ack.find(b'\x4B') != -1:
                     self._ser.timeout = timeout
-                    time.sleep(0.03)
                     self._shakehand_flag = True
+                    time.sleep(0.3)
                     return "OK"
             self._shakehand_flag = True
             self._ser.timeout = timeout
@@ -230,6 +242,10 @@ class BflbUartPort(object):
                     self._ser.setDTR(0)
                 bflb_utils.printf("power on tx and rx ")
                 time.sleep(0.1)
+            else:
+                self._ser.setDTR(0)
+                bflb_utils.printf("default set DTR high ")
+                time.sleep(0.1)
             if do_reset is True and blusbserialwriteflag is not True:
                 # MP_TOOL_V3 reset high to make boot pin high
                 self._ser.setRTS(0)
@@ -301,9 +317,14 @@ class BflbUartPort(object):
             bflb_utils.printf("send sync")
             # send keep 6ms ,N*10/baudrate=0.01
             if self._chiptype == "bl702":
-                self._ser.write(self._if_get_sync_bytes(int(0.003 * self._baudrate / 10)))
+                self._ser.write(self._if_get_sync_bytes(
+                    int(0.003 * self._baudrate / 10)))
             else:
-                self._ser.write(self._if_get_sync_bytes(int(0.006 * self._baudrate / 10)))
+                self._ser.write(self._if_get_sync_bytes(
+                    int(0.006 * self._baudrate / 10)))
+            if self._chipname == "bl808":
+                time.sleep(0.3)
+                self._ser.write(bflb_utils.hexstr_to_bytearray("5000080038F0002000000018"))
             if self._602a0_dln_fix:
                 time.sleep(4)
             success, ack = self.if_read(1000)
@@ -319,8 +340,8 @@ class BflbUartPort(object):
                 bflb_utils.printf("reshake")
                 if do_reset is False:
                     bflb_utils.printf("sleep")
-                    #tmp=0
-                    #while tmp<60:
+                    # tmp=0
+                    # while tmp<60:
                     #    time.sleep(2)
                     #    bflb_utils.printf("Pls check eflash loader log")
                     #    tmp+=1
@@ -331,6 +352,120 @@ class BflbUartPort(object):
         self._ser.timeout = timeout
         return "FL"
 
+    def if_toggle_boot(self,
+                     do_reset=False,
+                     reset_hold_time=100,
+                     shake_hand_delay=100,
+                     reset_revert=True,
+                     cutoff_time=0,
+                     shake_hand_retry=2,
+                     iap_timeout=0,
+                     boot_load=False):
+        timeout = self._ser.timeout
+        blusbserialwriteflag = False
+
+        if self.check_bl_usb_serial(self._device) and boot_load:
+            blusbserialwriteflag = True
+        # cut of tx rx power and rst
+        if cutoff_time != 0 and blusbserialwriteflag is not True:
+            cutoff_revert = False
+            if cutoff_time > 1000:
+                cutoff_revert = True
+                cutoff_time = cutoff_time - 1000
+            # MP_TOOL_V3 generate rising pulse to make D trigger output low
+            # reset low
+            self._ser.setRTS(1)
+            # RC delay is 100ms
+            time.sleep(0.2)
+            # reset high
+            self._ser.setRTS(0)
+            time.sleep(0.05)
+            # do power off
+            # reset low
+            self._ser.setRTS(1)
+            if cutoff_revert:
+                # dtr high, power off
+                self._ser.setDTR(0)
+            else:
+                # dtr low, power off
+                self._ser.setDTR(1)
+            bflb_utils.printf("tx rx and power off, press the machine!")
+            bflb_utils.printf("cutoff time is ", cutoff_time / 1000.0)
+            time.sleep(cutoff_time / 1000.0)
+            if cutoff_revert:
+                # dtr low, power on
+                self._ser.setDTR(1)
+            else:
+                # dtr high, power on
+                self._ser.setDTR(0)
+            bflb_utils.printf("power on tx and rx ")
+            time.sleep(0.1)
+        if do_reset is True and blusbserialwriteflag is not True:
+            # MP_TOOL_V3 reset high to make boot pin high
+            self._ser.setRTS(0)
+            time.sleep(0.2)
+            if reset_revert:
+                # reset low for reset revert to make boot pin high when cpu rset
+                self._ser.setRTS(1)
+                time.sleep(0.001)
+            reset_cnt = 2
+            if reset_hold_time > 1000:
+                reset_cnt = int(reset_hold_time // 1000)
+                reset_hold_time = reset_hold_time % 1000
+            while reset_cnt > 0:
+                if reset_revert:
+                    # reset high
+                    self._ser.setRTS(0)
+                else:
+                    # reset low
+                    self._ser.setRTS(1)
+                # Boot high
+                # self._ser.setDTR(0)
+                time.sleep(reset_hold_time / 1000.0)
+                if reset_revert:
+                    # reset low
+                    self._ser.setRTS(1)
+                else:
+                    # reset high
+                    self._ser.setRTS(0)
+                if shake_hand_delay > 0:
+                    time.sleep(shake_hand_delay / 1000.0)
+                else:
+                    time.sleep(5 / 1000.0)
+
+                # do reset agian to make sure boot pin is high
+                if reset_revert:
+                    # reset high
+                    self._ser.setRTS(0)
+                else:
+                    # reset low
+                    self._ser.setRTS(1)
+                # Boot high
+                # self._ser.setDTR(0)
+                time.sleep(reset_hold_time / 1000.0)
+                if reset_revert:
+                    # reset low
+                    self._ser.setRTS(1)
+                else:
+                    # reset high
+                    self._ser.setRTS(0)
+                if shake_hand_delay > 0:
+                    time.sleep(shake_hand_delay / 1000.0)
+                else:
+                    time.sleep(5 / 1000.0)
+                reset_cnt -= 1
+            bflb_utils.printf("reset cnt: " + str(reset_cnt) + ", reset hold: " +
+                                str(reset_hold_time / 1000.0) + ", shake hand delay: " +
+                                str(shake_hand_delay / 1000.0))
+        if blusbserialwriteflag:
+            self.bl_usb_serial_write(cutoff_time, reset_revert)
+        # clean buffer before start
+        bflb_utils.printf("clean buf")
+        self._ser.timeout = 0.1
+        self._ser.read_all()
+        self._ser.timeout = timeout
+        return "OK"
+
     def if_close(self):
         if self._ser is not None:
             self._ser.close()
@@ -343,19 +478,23 @@ class BflbUartPort(object):
             bflb_utils.printf("ack is ", str(binascii.hexlify(ack)))
             return ack.decode("utf-8")
         if ack.find(b'\x4F') != -1 or ack.find(b'\x4B') != -1:
-            #if dmy_data:
+            # if dmy_data:
             #    success,ack=self.if_read(14)
             #    if success==0:
             #        return "FL"
             return "OK"
+        elif ack.find(b'\x50') != -1 or ack.find(b'\x44') != -1:
+            return "PD"
         success, err_code = self.if_read(2)
         if success == 0:
             bflb_utils.printf("err code is ", str(binascii.hexlify(err_code)))
             return "FL"
-        err_code_str = str(binascii.hexlify(err_code[1:2] + err_code[0:1]).decode('utf-8'))
+        err_code_str = str(binascii.hexlify(
+            err_code[1:2] + err_code[0:1]).decode('utf-8'))
         ack = "FL"
         try:
-            ret = ack + err_code_str + "(" + bflb_utils.get_bflb_error_code[err_code_str] + ")"
+            ret = ack + err_code_str + \
+                "(" + bflb_utils.get_bflb_error_code(err_code_str) + ")"
         except Exception:
             ret = ack + err_code_str + " unknown"
         bflb_utils.printf(ret)
@@ -374,7 +513,8 @@ class BflbUartPort(object):
             data_len = bflb_utils.bytearray_to_int(tmp)
             success, data_bytes = self.if_read(data_len)
             if success == 0 or len(data_bytes) != data_len:
-                bflb_utils.printf("Read data error,maybe not get excepted length")
+                bflb_utils.printf(
+                    "Read data error,maybe not get excepted length")
                 # bflb_utils.printf("ack is ", str(binascii.hexlify(data_bytes)))
                 return "Read data error,maybe not get excepted length", data_bytes
             # bflb_utils.printf("ack is ", str(binascii.hexlify(data_bytes)))
@@ -386,8 +526,8 @@ class BflbUartPort(object):
 
 class CliInfUart(object):
 
-    def __init__(self, recv_cb_objs=None):
-        self._baudrate = 115200
+    def __init__(self, recv_cb_objs=None, baudrate=115200):
+        self._baudrate = baudrate
         self._ser = None
         # self._logfile = None
         self._rx_thread = None
@@ -428,7 +568,7 @@ class CliInfUart(object):
                     dev = dev_com[:dev_com.find(" (")]
                 else:
                     dev = dev_com
-                
+
                 self._ser = serial.Serial(dev,
                                           self._baudrate,
                                           timeout=5.0,
@@ -465,6 +605,7 @@ class CliInfUart(object):
 
     def close(self):
         if self._ser is not None:
+            self._ser.setDTR(1)
             self._ser.close()
             # self._logfile.close()
             self._ser = None
@@ -483,6 +624,10 @@ class CliInfUart(object):
             self._tx_queue.put(data_send)
         else:
             return 0
+        
+    def read(self):
+        time.sleep(0.1)
+        return self.msg
 
     def _write(self):
         while self._tx_thread_running:
@@ -523,7 +668,7 @@ class CliInfUart(object):
 
     def _read(self):
         data = ''
-
+        self.msg = ''
         while self._rx_thread_running:
             try:
                 byte_msg = self._ser.read(1)
@@ -546,6 +691,7 @@ class CliInfUart(object):
                     continue
                 if str(str_msg) == '\r':
                     if data != '':
+                        self.msg = data
                         bflb_utils.printf(data)
                         # check on_boot
                         if self._check_boot_cond(data) is True:
