@@ -35,6 +35,7 @@ import hashlib
 import argparse
 import traceback
 import platform
+import codecs
 from glob import glob
 
 import pylink
@@ -42,30 +43,35 @@ from serial import Serial
 from Crypto.Util import Counter
 from Crypto.Cipher import AES
 
-import globalvar as gol
 try:
     from PySide2 import QtCore
+    qt_sign = True
 except ImportError:
     qt_sign = False
-    
+
+try:
+    from serial.tools.list_ports import comports
+except ImportError:
+    raise exception.GetSerialPortsError(os.name)
+
 # Get app path
 if getattr(sys, "frozen", False):
     app_path = os.path.dirname(sys.executable)
 else:
     app_path = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
- 
+chip_path = os.path.join(app_path, "chips")
     
 python_version = struct.calcsize("P") * 8
-
 if python_version == 64:
-    path_dll = os.path.join(app_path, "JLink_x64.dll")
+    path_dll = os.path.join(app_path, "utils/jlink", "JLink_x64.dll")
 else:
-    path_dll = os.path.join(app_path, "JLinkARM.dll")
+    path_dll = os.path.join(app_path, "utils/jlink", "JLinkARM.dll")
 
 try:
     import changeconf as cgc
     conf_sign = True
 except ImportError:
+    cgc = None
     conf_sign = False
     
 PY2 = sys.version_info[0] == 2
@@ -75,9 +81,24 @@ udp_send_log = False
 udp_log_local_echo = False
 udp_socket_server = None
 error_code_num = "FFFF"
+error_code_num_task = ["FFFF", "FFFF", "FFFF", "FFFF", "FFFF", \
+                       "FFFF", "FFFF", "FFFF", "FFFF", "FFFF", \
+                       "FFFF", "FFFF", "FFFF", "FFFF", "FFFF", \
+                       "FFFF", "FFFF", "FFFF", "FFFF", "FFFF", \
+                       "FFFF", "FFFF", "FFFF", "FFFF", "FFFF", \
+                       "FFFF", "FFFF", "FFFF", "FFFF", "FFFF", \
+                       "FFFF", "FFFF", "FFFF", "FFFF", "FFFF", \
+                       "FFFF", "FFFF", "FFFF", "FFFF", "FFFF", \
+                       "FFFF", "FFFF", "FFFF", "FFFF", "FFFF", \
+                       "FFFF", "FFFF", "FFFF", "FFFF", "FFFF", \
+                       "FFFF", "FFFF", "FFFF", "FFFF", "FFFF", \
+                       "FFFF", "FFFF", "FFFF", "FFFF", "FFFF", \
+                       "FFFF", "FFFF", "FFFF", "FFFF", "FFFF", \
+                       "FFFF"]
 local_log_en = True
 local_log_data = ""
 
+
 # all in hex mode
 if conf_sign:
     bflb_error_code = {
@@ -94,6 +115,7 @@ if conf_sign:
         "000a": "FLASH_WRITE_STATUS_REG_ERROR",
         "000b": "FLASH_DECOMPRESS_WRITE_ERROR",
         "000c": "FLASH_WRITE_XZ_ERROR",
+        "000d": "FLASH_SWITCH_BANK_ERROR",
         "0101": "CMD_ID_ERROR",
         "0102": "CMD_LEN_ERROR",
         "0103": "CMD_CRC_ERROR",
@@ -137,6 +159,24 @@ if conf_sign:
         "0406": "EFUSE_READ_ERROR",
         "0407": "EFUSE_READ_MAC_ERROR",
         "0408": "EFUSE_WRITE_MAC_ERROR",
+        "0501": "MEMORY_WRITE_PARA_ERROR",
+        "0502": "MEMORY_WRITE_ADDR_ERROR",
+        "0503": "MEMORY_WRITE_ERROR",
+        "0504": "MEMORY_READ_PARA_ERROR",
+        "0505": "MEMORY_READ_ADDR_ERROR",
+        "0506": "MEMORY_READ_ERROR",
+        "0508": "REG_WRITE_PARA_ERROR",
+        "0509": "REG_WRITE_ADDR_ERROR",
+        "050a": "REG_WRITE_ERROR",
+        "050b": "REG_READ_PARA_ERROR",
+        "050c": "REG_READ_ADDR_ERROR",
+        "050d": "REG_READ_ERROR",
+        "0601": "ECDH_PARA_ERROR",
+        "0602": "ECDH_PRIVATE_KEY_ERROR",
+        "0603": "ECDH_SHARED_KEY_ERROR",
+        "0604": "ECDH_RANDOM_VAL_ERROR",
+        "0605": "ECDH_DECRYPT_ERROR",
+        "0606": "ECDH_ENCRYPT_ERROR",
         "fffc": "PLL_ERROR",
         "fffd": "INVASION_ERROR",
         "fffe": "POLLING",
@@ -155,6 +195,8 @@ if conf_sign:
         "000A": "REPEAT BURN",
         "000B": "CONFIG FILE NOT FOUND",
         "000C": "SET CLOCK PLL FAIL",
+        "000D": "SET OPT FINISH FAIL",
+        "000E": "IMPORT PACKET FAIL",
         "0020": "EFUSE READ FAIL",
         "0021": "EFUSE WRITE FAIL",
         "0022": "EFUSE COMPARE FAIL",
@@ -178,6 +220,9 @@ if conf_sign:
         "003E": "FLASH LOAD VERIFY FAIL",
         "003F": "FLASH BOOT FAIL",
         "0040": "FLASH CFG NOT FIT WITH BOOTHEADER",
+        "0041": "FLASH LOAD ROMFS FAIL",
+        "0042": "FLASH SWITCH BANK FAIL",
+        "0043": "FLASH IDENTIFY FAIL",
         "0050": "IMG LOAD SHAKEHAND FAIL",
         "0051": "IMG LOAD BOOT CHECK FAIL",
         "0060": "IMG CREATE FAIL",
@@ -209,6 +254,12 @@ if conf_sign:
         "0080": "ROMFS NOT SET",
         "0081": "MFG BIN NOT SET",
         "0082": "PT CHECK FAIL",
+        "0083": "D0 FW BIN NOT SET",  
+        "0084": "IMTB BIN NOT SET", 
+        "0085": "IMG LOADER BIN NOT SET",
+        "0086": "SBI BIN NOT SET",
+        "0087": "KERNEL BIN NOT SET",  
+        "0088": "ROOTFS BIN NOT SET",     
         "FFFF": "BURN RETRY FAIL",
     }
 else:
@@ -226,6 +277,7 @@ else:
         "000a": "BFLB_FLASH_WRITE_STATUS_REG_ERROR",
         "000b": "BFLB_FLASH_DECOMPRESS_WRITE_ERROR",
         "000c": "BFLB_FLASH_WRITE_XZ_ERROR",
+        "000d": "BFLB_FLASH_SWITCH_BANK_ERROR",
         "0101": "BFLB_CMD_ID_ERROR",
         "0102": "BFLB_CMD_LEN_ERROR",
         "0103": "BFLB_CMD_CRC_ERROR",
@@ -269,6 +321,24 @@ else:
         "0406": "BFLB_EFUSE_READ_ERROR",
         "0407": "BFLB_EFUSE_READ_MAC_ERROR",
         "0408": "BFLB_EFUSE_WRITE_MAC_ERROR",
+        "0501": "BFLB_MEMORY_WRITE_PARA_ERROR",
+        "0502": "BFLB_MEMORY_WRITE_ADDR_ERROR",
+        "0503": "BFLB_MEMORY_WRITE_ERROR",
+        "0504": "BFLB_MEMORY_READ_PARA_ERROR",
+        "0505": "BFLB_MEMORY_READ_ADDR_ERROR",
+        "0506": "BFLB_MEMORY_READ_ERROR",
+        "0508": "BFLB_REG_WRITE_PARA_ERROR",
+        "0509": "BFLB_REG_WRITE_ADDR_ERROR",
+        "050a": "BFLB_REG_WRITE_ERROR",
+        "050b": "BFLB_REG_READ_PARA_ERROR",
+        "050c": "BFLB_REG_READ_ADDR_ERROR",
+        "050d": "BFLB_REG_READ_ERROR",
+        "0601": "BFLB_ECDH_PARA_ERROR",
+        "0602": "BFLB_ECDH_PRIVATE_KEY_ERROR",
+        "0603": "BFLB_ECDH_SHARED_KEY_ERROR",
+        "0604": "BFLB_ECDH_RANDOM_VAL_ERROR",
+        "0605": "BFLB_ECDH_DECRYPT_ERROR",
+        "0606": "BFLB_ECDH_ENCRYPT_ERROR",
         "fffc": "BFLB_PLL_ERROR",
         "fffd": "BFLB_INVASION_ERROR",
         "fffe": "BFLB_POLLING",
@@ -287,6 +357,8 @@ else:
         "000A": "BFLB REPEAT BURN",
         "000B": "BFLB CONFIG FILE NOT FOUND",
         "000C": "BFLB SET CLOCK PLL FAIL",
+        "000D": "BFLB SET OPT FINISH FAIL",
+        "000E": "BFLB IMPORT PACKET FAIL",
         "0020": "BFLB EFUSE READ FAIL",
         "0021": "BFLB EFUSE WRITE FAIL",
         "0022": "BFLB EFUSE COMPARE FAIL",
@@ -310,6 +382,9 @@ else:
         "003E": "BFLB FLASH LOAD VERIFY FAIL",
         "003F": "BFLB FLASH BOOT FAIL",
         "0040": "BFLB FLASH CFG NOT FIT WITH BOOTHEADER",
+        "0041": "BFLB FLASH LOAD ROMFS FAIL",
+        "0042": "BFLB FLASH SWITCH BANK FAIL",
+        "0043": "BFLB FLASH IDENTIFY FAIL",
         "0050": "BFLB IMG LOAD SHAKEHAND FAIL",
         "0051": "BFLB IMG LOAD BOOT CHECK FAIL",
         "0060": "BFLB IMG CREATE FAIL",
@@ -341,6 +416,12 @@ else:
         "0080": "BFLB ROMFS NOT SET",
         "0081": "BFLB MFG BIN NOT SET",
         "0082": "BFLB PT CHECK FAIL",
+        "0083": "D0 FW BIN NOT SET",  
+        "0084": "IMTB BIN NOT SET",   
+        "0085": "IMG LOADER BIN NOT SET",
+        "0086": "SBI BIN NOT SET",
+        "0087": "KERNEL BIN NOT SET",  
+        "0088": "ROOTFS BIN NOT SET",      
         "FFFF": "BFLB BURN RETRY FAIL",
     }
 
@@ -356,6 +437,9 @@ def printf(*args):
     # print(data.title())
     # print(data.capitalize())
     if data:
+        if conf_sign:
+            for key, value in cgc.replace_name_list.items():
+                data = data.replace(key, value)
         now_time = datetime.datetime.now().strftime('[%H:%M:%S.%f')[:-3] + '] - '
         data = now_time + data
 
@@ -376,8 +460,7 @@ def printf(*args):
             except Exception as e:
                 print(e)
         else:
-            qt_sign = gol.GlobalVar.qt
-            if qt_sign:
+            if qt_sign and QtCore.QThread.currentThread().objectName():
                 print("[Task%s]" % str(QtCore.QThread.currentThread().objectName()) + data.strip())
             else:
                 print(data.strip())
@@ -404,9 +487,11 @@ def local_log_save(local_path="log", key_word=""):
             rq = time.strftime('%Y%m%d%H%M%S', time.localtime(time.time()))
             log_name = rq + '_' + key_word + '.log'
             log_path = os.path.join(log_dir, log_name)
-            fp = open(log_path, 'w')
-            fp.write(local_log_data)
-            fp.close()
+#             fp = open(log_path, 'w')
+#             fp.write(local_log_data)
+#             fp.close()
+            with codecs.open(log_path, "w", encoding="utf-8") as fp:
+                fp.write(local_log_data)
         except Exception as e:
             printf(e)
             traceback.print_exc(limit=5, file=sys.stdout)
@@ -418,22 +503,38 @@ def get_bflb_error_code(code):
     return bflb_error_code[code]
 
 
-def set_error_code(num_str):
+def set_error_code(num_str, task=None):
     global error_code_num
-    if error_code_num == "FFFF":
-        error_code_num = num_str
-    if num_str == "FFFF":
-        error_code_num = num_str
+    global error_code_num_task
+    if task != None:
+        if len(error_code_num_task) == 0:
+            for i in range(66):
+                error_code_num_task.append("FFFF")
+        if error_code_num_task[task] == "FFFF":
+            error_code_num_task[task] = num_str
+        if num_str == "FFFF":
+            error_code_num_task[task] = num_str
+    else:
+        if error_code_num == "FFFF":
+            error_code_num = num_str
+        if num_str == "FFFF":
+            error_code_num = num_str
 
 
-def get_error_code():
+def get_error_code(task=None):
     global error_code_num
+    global error_code_num_task
+    if task != None:
+        return error_code_num_task[task]
     return error_code_num
 
 
-def errorcode_msg():
+def errorcode_msg(task=None):
     global error_code_num
+    global error_code_num_task
     global eflash_loader_error_code
+    if task != None:
+        return '{"ErrorCode": "' + error_code_num_task[task] + '", "ErrorMsg":"' + eflash_loader_error_code[error_code_num_task[task]] + '"}'
     return '{"ErrorCode": "' + error_code_num + '", "ErrorMsg":"' + eflash_loader_error_code[error_code_num] + '"}'
 
 
@@ -582,6 +683,17 @@ def img_create_encrypt_data(data_bytearray, key_bytearray, iv_bytearray, flash_i
     return ciphertext
 
 
+def aes_decrypt_data(data, key_bytearray, iv_bytearray, flash_img):
+    if flash_img == 0:
+        cryptor = AES.new(key_bytearray, AES.MODE_CBC, iv_bytearray)
+        plaintext = cryptor.decrypt(data)
+    else:
+        iv = Counter.new(128, initial_value=int(binascii.hexlify(iv_bytearray), 16))
+        cryptor = AES.new(key_bytearray, AES.MODE_CTR, counter=iv)
+        plaintext = cryptor.decrypt(data)
+    return plaintext
+
+
 def open_file(file, mode='rb'):
     fp = open(os.path.join(app_path, file), mode)
     return fp
@@ -635,63 +747,104 @@ def get_serial_ports():
 
 
 def serial_enumerate():
+    prog_ports = []
+    sdio_ports = []
+    sdio_file_ser_dict = {}
+    uart_ports = []
+    file_dict = {}
     ports = []
     if sys.platform.startswith("win"):
+        for p, d, h in comports():
+            if "Virtual" in d or not p:
+                if "STM" not in d:
+                    continue 
+            if "PID=1D6B" in h.upper():
+                ser_value = h.split(" ")[2][4:]
+                if ser_value not in sdio_file_ser_dict:
+                    #sdio_ports.append(p+" (SDIO)")
+                    sdio_file_ser_dict[ser_value] = p
+                else:
+                    if "LOCATION" in h.upper():
+                        file_dict[sdio_file_ser_dict[ser_value]] = p
+                        sdio_ports.append(sdio_file_ser_dict[ser_value]+" (SDIO)")
+                    else:
+                        file_dict[p] = sdio_file_ser_dict[ser_value]
+                        sdio_ports.append(p+" (SDIO)")
+            else:
+                if "FACTORYAIOT_PROG" in h.upper() or "PID=42BF:B210" in h.upper():
+                    prog_ports.append(p+" (PROG)")
+                else:
+                    uart_ports.append(p)
         try:
-            from serial.tools.list_ports import comports
-        except ImportError:
-            raise exception.GetSerialPortsError(os.name)        
-        ports = []
+            uart_ports = sorted(uart_ports, key=lambda x: int(re.match('COM(\d+)', x).group(1)))
+        except Exception:
+            uart_ports = sorted(uart_ports)                    
+        ports = sorted(prog_ports) + sorted(sdio_ports) + uart_ports
+    elif sys.platform.startswith('linux'):
         for p, d, h in comports():
             if not p:
-                continue
-            if "VID:PID" in h:
-                ports.append(p)
-        
-# Iterate through registry because WMI does not show virtual serial ports
-#         import winreg
-#         try:
-#             key = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, r'HARDWARE\DEVICEMAP\SERIALCOMM')
-#         except WindowsError:
-#             return []
-#         i = 0
-#         while True:
-#             try:
-#                 # ports.append([winreg.EnumValue(key, i)[0], winreg.EnumValue(key, i)[1]])
-#                 ports.append(winreg.EnumValue(key, i)[1])
-#                 i = i + 1
-#             except WindowsError:
-#                 break
-    elif sys.platform.startswith('linux'):
-        if os.path.exists('/dev/serial/by-id'):
-            entries = os.listdir('/dev/serial/by-id')
-            dirs = [os.readlink(os.path.join('/dev/serial/by-id', x)) for x in entries]
-            ports.extend([os.path.normpath(os.path.join('/dev/serial/by-id', x)) for x in dirs])
-        if os.path.exists('/dev/serial/by-path'):
-            entries = os.listdir('/dev/serial/by-path')
-            dirs = [os.readlink(os.path.join('/dev/serial/by-path', x)) for x in entries]
-            ports.extend([os.path.normpath(os.path.join('/dev/serial/by-path', x)) for x in dirs])
-        for dev in glob('/dev/ttyS*'):
-            try:
-                port = Serial(dev)
-            except Exception:
-                pass
+                continue        
+            if "PID=1D6B" in h.upper():
+                ser_value = h.split(" ")[2][4:]
+                if ser_value not in sdio_file_ser_dict:
+                    sdio_file_ser_dict[ser_value] = p
+                else:
+                    if sdio_file_ser_dict[ser_value]>p:
+                        file_dict[p] = sdio_file_ser_dict[ser_value]
+                        sdio_ports.append(p+" (SDIO)")
+                    else:
+                        file_dict[sdio_file_ser_dict[ser_value]] = p
+                        sdio_ports.append(sdio_file_ser_dict[ser_value]+" (SDIO)")
             else:
-                ports.append(dev)
+                if "FACTORYAIOT PROG" in h.upper():
+                    prog_ports.append(p+" (PROG)")
+                else:
+                    uart_ports.append(p)
+        ports = sorted(prog_ports) + sorted(sdio_ports) + sorted(uart_ports)
     elif sys.platform.startswith('darwin'):
-        for dev in glob('/dev/tty.usbserial-*'):
+        for dev in glob('/dev/tty.usb*'):
             ports.append(dev)
-    else:
-        return []
-    return list(set(ports))
+    return ports
 
 
 def pylink_enumerate():
-    obj_dll = pylink.Library(dllpath=path_dll)
-    obj = pylink.JLink(lib=obj_dll)
-    return obj.connected_emulators()
-
-
+    try:
+        if sys.platform == 'win32':
+            obj_dll = pylink.Library(dllpath=path_dll)
+            obj = pylink.JLink(lib=obj_dll)
+        else:
+            obj = pylink.JLink()
+    except Exception:
+        return []
+    else:
+        return obj.connected_emulators()
+      
+      
+def cklink_openocd_enumerate():
+    ports_cklink = []
+    ports_openocd = []
+    if sys.platform.startswith("win"):     
+        for p, d, h in comports():
+            if not p:
+                continue
+            elif "FACTORYAIOT_PROG" in h.upper():
+                match1 = re.search("FACTORYAIOT_PROG_([a-zA-Z0-9]{6}) LOCATION", h.upper(), re.I)
+                match2 = re.search("FACTORYAIOT_PROG_([a-zA-Z0-9]{6})$", h.upper(), re.I)
+                if match1 is not None:
+                    ports_cklink.append(match1.group(1)) 
+                if match2 is not None:
+                    ports_openocd.append(match2.group(1)) 
+    elif sys.platform.startswith("linux"):     
+        for p, d, h in comports():
+            if not p:
+                continue
+            elif "FactoryAIOT Prog" in h:
+                match1 = re.search("FactoryAIOT Prog ([a-zA-Z0-9]{6}) LOCATION", h, re.I)
+                if match1 is not None:
+                    ports_cklink.append(match1.group(1)) 
+    return ports_cklink, ports_openocd
+                 
+                 
 def image_create_parser_init():
     parser = argparse.ArgumentParser(description="bouffalolab image create command")
     parser.add_argument("--chipname", dest="chipname", help="chip name")
@@ -710,6 +863,7 @@ def eflash_loader_parser_init():
     parser.add_argument("--usage", dest="usage", action="store_true", help="display usage")
     parser.add_argument("--flash", dest="flash", action="store_true", help="target is flash")
     parser.add_argument("--efuse", dest="efuse", action="store_true", help="target is efuse")
+    parser.add_argument("--ram", dest="ram", action="store_true", help="target is ram")
     parser.add_argument("-w", "--write", dest="write", action="store_true", help="write to flash/efuse")
     parser.add_argument("-e", "--erase", dest="erase", action="store_true", help="erase flash")
     parser.add_argument("-r", "--read", dest="read", action="store_true", help="read from flash/efuse")
@@ -724,15 +878,18 @@ def eflash_loader_parser_init():
     parser.add_argument("--addr", dest="addr", help="address to write")
     parser.add_argument("--mac", dest="mac", help="mac address to write")
     parser.add_argument("--file", dest="file", help="file to store read data or file to write")
+    parser.add_argument("--packet", dest="packet", help=" import packet to replace burn file")
     parser.add_argument("--efusefile", dest="efusefile", help="efuse file to write efuse")
     parser.add_argument("--data", dest="data", help="data to write")
     parser.add_argument("--mass", dest="mass", help="load mass bin")
     parser.add_argument("--loadstr", dest="loadstr", help="")
     parser.add_argument("--loadfile", dest="loadfile", help="")
     parser.add_argument("--userarea", dest="userarea", help="user area")
+    parser.add_argument("--romfs", dest="romfs", help="romfs data to write")
     parser.add_argument("--csvfile", dest="csvfile", help="csv file contains 3/5 tuples")
     parser.add_argument("--csvaddr", dest="csvaddr", help="address to write for csv file")
     parser.add_argument("--para", dest="para", help="efuse para")
+    parser.add_argument("--iap", dest="iap", action="store_true", help="iap page config")
     parser.add_argument("--createcfg", dest="createcfg", help="img create cfg file")
     parser.add_argument("--key", dest="key", help="aes key for socket")
     parser.add_argument("--ecdh", dest="ecdh", action="store_true", help="open ecdh function")
