@@ -20,15 +20,7 @@
 #  SOFTWARE.
 
 
-try:
-    import changeconf as cgc
-    conf_sign = True
-except ImportError:
-    conf_sign = False
+version_text = "(V_2.4.0)"
+eflash_loader_version_text = "eflash_loader_v2.4.0"
 
-version_text = "(V_2.3.0)"
-if conf_sign:
-    eflash_loader_version_text = "eflash_loader_v2.3.0"
-else:
-    eflash_loader_version_text = "bflb_eflash_loader_v2.3.0"
-    
+    
\ No newline at end of file
